%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Program do zobrazowania wynikw DPF - przesunicie koowe i liniowe
% sygnau dyskretnego
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Program obrazuje efekty przesunicia sygnau w dziedzinie czasu
% obserwowane w jego dyskretnej transformacie Fouriera.
% Moliwe jest porwnanie liniowego i koowego przesunicia w czasie.
% Wyboru rodzaju przesunicia dokonuje si za pomoc staej PRZESUNIECIE.
% Warto przesunicia zmienia si za pomoc staej n0, moe ona przyjmowa
% wartoci dodatnie i ujemne.
% Moliwy jest wybr sygnau o postaci:
% 1 - sygna liniowo narastajcy x(n)=n/N
% 2 - delta Kroneckera
% 3 - prostoktny impuls wizyjny
% Wyboru postaci sygnau dokonuje si za pomoc staej SYGNAL.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all;
clear;
clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wybr parametrw programu
% posta sygnau: 
% 1 - sygna liniowo narastajcy x(n)=n/N
% 2 - delta Kroneckera
% 3 - prostoktny impuls wizyjny
% dla n nalecych do <0,N-1>
sygnal=1;
% rodzaj przesunicia sygnau w czasie
% 1 - liniowe
% 2 - koowe
PRZESUNIECIE=2;
% przesunicie sygnau
n0=-10;
% ilo prbek 
N=127;
% wektor prbek czstotliwociowych
k=0:N-1;
% taki sam wektor prbek czasu
n=k;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% utworzenie cigu prbek sygnau
switch sygnal
    case 1
        % sygna liniowo narastajcy x(n)=n/N
        tytul=' sygnau liniowo narastajcego ';
        x=(1:N)/N;
    case 2
        % delta Kroneckera
        tytul=' delty Kroneckera ';
        x=zeros(1,N);
        x(1)=1;
    case 3
        % impuls prostoktny
        % dugo impulsu
        dlImp=16;
        tytul=' prostoktnego impulsu wizyjnego ';
        x=zeros(1,N);
        x(1:dlImp)=1;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% dokonujemy przesunicia
switch PRZESUNIECIE
    case 1
        % przesunicie liniowe
        y=zeros(1,N+abs(n0));
        if(n0>0)
            % przesunicie w prawo - n0 dodatnie
            y(1,n0+1:N+n0)=x;
            x=y(1,1:N);
        else
            % przesunicie w lewo - n0 ujemne
            y(1,1:N)=x;
            x=y(1,abs(n0)+1:N+abs(n0));
        end;
        rodzaj=' liniowym ';
    case 2
        % przesunicie koowe
        x=circshift(x, [0 n0]);
        rodzaj=' koowym ';
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% wyznaczamy widmo
W=fft(x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% wykres postaci czasowej sygnau
fig1=figure;
fig1.WindowState='maximized';
subplot(3,1,1);
stem(k,x,'b','filled','LineWidth',1.2,'MarkerSize',3);
xlim([0 N-1]);
ylim([0 1.2]);
title(['Posta czasowa' tytul 'przy przesuniciu' rodzaj 'o ' num2str(n0) ' prbek']);
xlabel('Numer prbki czasu n');
ylabel('Warto prbki x(n)');
% wykres widma amplitudowego sygnau
subplot(3,1,2);
stem(k,abs(W),'b','filled','LineWidth',1.2,'MarkerSize',3);
title(['Widmo amplitudowe' tytul 'przy przesuniciu' rodzaj 'o ' num2str(n0) ' prbek']);
xlabel('Numer prbki czstotliwociowej k');
ylabel('Warto prbki X(k)');
% wykres widma fazowego sygnau
subplot(3,1,3);
stem(k,angle(W),'b','filled','LineWidth',1.2,'MarkerSize',3);
title(['Widmo fazowe' tytul 'przy przesuniciu' rodzaj 'o ' num2str(n0) ' prbek']);
xlabel('Numer prbki czstotliwociowej k');
ylabel('Warto prbki X(k)');
% koniec programu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%