%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Program do zobrazowania wynikw DPF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Program porwnuje widma wyznaczone poprzez analityczne obliczenie DPF
% z sygnau wykadniczego oraz przez wbudowan funkcj Matlaba: fft.
% Rnice pomidzy wynikami s rzdu 1e-15
% Moliwe postacie sygnau to:
% - rzeczywisty sygna malejcy wykadnicz0 a^n - w przypadku zmiany 
%   wartoci a na wiksz od jednoci, sygna bdzie narasta wykadniczo
% - zespolony sygna wykadniczy exp^(2i*pi*10*n/N) - zespolony 
%   sygna harmoniczny o staej amplitudzie rwnej 1
% - zespolony sygna wykadniczy o wykadniczo malejcej amplitudzie
% Wyboru postaci sygnau dokonuje si za pomoc staej SYGNAL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all;
clear;
clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% posta sygnau zespolonego
SYGNAL=0;
% moliwe opcje:
% 0 - rzeczywisty sygna wykadniczy a^n
% 1 - zespolony sygna wykadniczy exp^(2i*pi*10*n/N) - 
%                                       zespolony sygna harmoniczny
% 2 - zespolony sygna wykadniczy o wykadniczo malejcej amplitudzie
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ilo prbek sygnau
N=128;
% wektor prbek czstotliwociowych
k=0:N-1;
% taki sam wektor prbek czasu
n=k;
% posta sygnau x(n)=a^n dla N nalecego do <0;N-1>
switch SYGNAL
    case 0
        % rzeczywisty sygna wykadniczy a^n
        a=0.6;
    case 1
        % zespolony sygna wykadniczy exp^(2i*pi*10*n/N)
        % czstotliwo sygnau harmonicznego
        f0=10;
        a=exp(2i*pi*f0/N);
    case 2
        % zespolony sygna wykadniczy o wykadniczo malejcej amplitudzie
        % exp^(2i*pi*10*n/N)
        % czstotliwo sygnau harmonicznego
        f0=16;
        a=0.95*exp(2i*pi*f0/N);
end;
% posta czasowa sygnau
x=a.^n;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wykres postaci czasowej sygnau
fig1=figure;
fig1.WindowState='maximized';
% w oglnym przypadku sygna moe by zespolony, wic rysujemy cz
% rzeczywist i urojon
% cz rzeczywista
subplot(3,1,1);
title('Posta czasowa sygnau dyskretnego');
plot(n,real(x),'b-x');
hold on;
plot(n,imag(x),'r-x');
title('Posta czasowa sygnau dyskretnego');
legend('[Re[x(n)]','Im[x(n)]');
xlabel('nr prbki czasu n');
ylabel('Warto prbki x(n)');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wyznaczenie widma za pomoc funkcji fft
W=fft(x);
% posta widma wyznaczona w sposb analityczny
W1=(1-a^N)./(1-a*exp(-1i*2*pi*k/N));
% wykres widma amplitudowego
subplot(3,1,2);
stem(k,abs(W),'b','filled','LineWidth',1.2,'MarkerSize',3);
title('Widmo wyznaczone w sposb analityczny');
maksW=max(max(abs(W)));
xlim([0,N-1]);
ylim([0 1.2*maksW]);
xlabel('Numer prbki czstotliwociowej k');
ylabel('Warto prbki X(k)');
% wykres rnicy widm
subplot(3,1,3);
stem(abs(W1-W),'b','filled','LineWidth',1.2,'MarkerSize',3);
title('Rnica midzy widmem wyznaczonym analitycznie oraz fft');
xlabel('Numer prbki czstotliwociowej k');
ylabel('Warto prbki rnicowej \DeltaX(k)');
% koniec programu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%